;;;Script is a mashup of two scripts: 
;;;Guillaume de Sercey (gd32@bton.ac.uk) - Text Along Path 
;;;Arch. Giuseppe Conte - Draw Circle 
;;;Thank you 
;;;Script is supposed to generate circles along a given path
;;;Optional behavior is to fill these circles with a shade of black based on the path orientation

(define (script-fu-path-point-at-dist img path stroke position atlast)
  (let*
    (
			 (delta 0.1)
       (deltastart 0.2)
       (point (gimp-vectors-stroke-get-point-at-dist path stroke position 1))
       (ang (atan (caddr point)))    ;this return an angle between [-pi,+pi] and needs to be adjusted
       (point1 (butlast (gimp-vectors-stroke-get-point-at-dist path stroke (- position deltastart) 1)))
			 (point2)
    );end declare
		(if (= atlast 1)
			(set! point2 (butlast (gimp-vectors-stroke-get-point-at-dist path stroke (+ position deltastart) 1))) 
			(set! point2 (butlast (gimp-vectors-stroke-get-point-at-dist path stroke (+ position 0.00005) 1)))
		)
    (while (equal? point1 point2)
			(set! deltastart (+ deltastart delta))
			(set! point1 (butlast (gimp-vectors-stroke-get-point-at-dist path stroke (- position deltastart) 1)))
			(if (= atlast 1)
					(set! point2 (butlast (gimp-vectors-stroke-get-point-at-dist path stroke (+ position deltastart) 1))) 
					(set! point2 (butlast (gimp-vectors-stroke-get-point-at-dist path stroke (+ position 0.00005) 1)))
			)
		)
		(if (= (car point1) (car point2));if x coords are equal
			(if (< (cadr point1) (cadr point2)) ;First point is below the second
				(set! ang *pi*)										;Raising
				(set! ang 0)											;Sinking
			)
			(if (= (cadr point1) (cadr point2));if y coords are equal
				(if (< (car point1) (car point2)) ;First point is more to the left than the second
					(set! ang (/ *pi* 2))						;Going Right
					(set! ang (* 1.5 *pi*))					;Going Left
				)
				(if (> (car point1) (car point2))	;True: 2 and 4 Quadrant; False:1 and 3 Quadrant
					(if (< (cadr point1) (cadr point2))
						(set! ang (+ (* 1.5 *pi*) (abs ang)))				;1 QUADRANT
						(set! ang (+ *pi* (abs(- ang (/ *pi* 2)))))	;3 QUADRANT
						;(set! ang (+ (* 1.5 *pi*) (abs ang)))	;THIRD QUADRANT
					)
					(if (> (cadr point1) (cadr point2))
						(set! ang (abs (- ang (/ *pi* 2))))		;2QUADRANT
						(set! ang (abs (- ang (/ *pi* 2))))		;4 QUADRANT						
					)
				)
			)
		);end if x coords equal
    (list (car point) (cadr point) ang)
  )
)

;;; Function to draw a circle at a specified point
(define (script-fu-circles-along-path img layer opt fcolor radius segments toggle angleo)
	(let*
		(
			(shade (car(gimp-context-get-foreground)))
		  (position 0)
		  (sx 0)
		  (sy 0)
		  (dx 0)
		  (dy 0)
		  (i 0)
			(shade)
			(point '(0 0 0))
			(diameter)
			(vectors)
		  (path)
		  (pathlen)
		  (segment)
			(feather 5)
			(anglec)
			(atlast 0)
		);end variable declaration
		(set! vectors (car (gimp-image-get-vectors img)))
		(if (<= vectors 0)
			(gimp-message "Please create and/or select a path first!")
			(
				(set! path (car (gimp-image-get-active-vectors img)))
			  (set! pathlen (- (car (gimp-vectors-stroke-get-length path 1 1)) 0.0005))
				(print pathlen)
				(set! angleo (/ (* angleo *pi*) 180))
			  (if (= toggle 1)
					(begin
						(set! segment (* 1.6 radius))
						(set! feather (/ radius 5))
						(set! segments (/ pathlen segment))
					)
					(set! segment (/ pathlen segments))
				)	
				(while (< i segments) ;change to: lower or EQUAL to
					(if (= i (- segments 1))
						(begin
							(set! position (- pathlen 0.0001))
							(set! atlast 1)
						)
					)
					(set! point (script-fu-path-point-at-dist img path 1 position atlast))
					(set! sx (- (car point) radius))
					(set! sy (- (cadr point) radius))
					(set! diameter (* radius 2))
					(set! dx diameter)
					(set! dy diameter)
					(gimp-ellipse-select img sx sy dx dy REPLACE 1 1 feather)
					(if (= opt 0)
						(begin
							(set! anglec (+ (caddr point) angleo))
							(if (> anglec (* 2 *pi*))
								(set! anglec (- anglec (* 2 *pi*)))
								(if (< anglec 0)
									(set! anglec (+ anglec (* 2 *pi*)))
								)
							)
							(set! shade (/ (* 255 anglec) (* 2 *pi*))) ;the actual angle 'adjusted' by the angle specified is converted into a shade of gray
							(set! shade (list shade shade shade))
							;(print (caddr point))
							;(print anglec)
						);end begin
						(if (= opt 2)
							(set! shade fcolor)
						)
					)
					;(print r) ;used for debugging to display the current lightness value for each point
					(gimp-context-set-foreground shade)
					(gimp-edit-fill layer 0)
					(set! i (+ i 1))
					(set! position (+ position segment))
				);end while
				(gimp-displays-flush)
				(gimp-selection-none img)
			)
		);end if
  );let
);define

(script-fu-register
	"script-fu-circles-along-path"
	"<Image>/Filters/Render/Circles@Path"
	"Renders circles of specified size along a path"
	"Lukas Cesnak"
	"Slovakia"
	"2007-11-04"

	"RGB* GRAY* INDEXED*"

SF-IMAGE "The Image" 0
SF-DRAWABLE "The Layer" 0
SF-OPTION	"Use color" '("By Orientation" "Foreground" "Specified below")
SF-COLOR "Specific Color" '(123 123 123)
SF-ADJUSTMENT "Radius" '(20 1 100 1 10 1 0)
;SF-ADJUSTMENT "Radius" '(20 0 100 1 10 0 1)
SF-ADJUSTMENT "Segments" '(20 1 100 1 10 1 0)
SF-TOGGLE     "Segments by radius" TRUE
SF-ADJUSTMENT "Angle adjustment" '(0 -180 180 1 10 0 1)
)